diff --git a/languagetool-core/src/main/java/org/languagetool/JLanguageTool.java b/languagetool-core/src/main/java/org/languagetool/JLanguageTool.java
index 04497b8..6e9f04a 100644
--- a/languagetool-core/src/main/java/org/languagetool/JLanguageTool.java
+++ b/languagetool-core/src/main/java/org/languagetool/JLanguageTool.java
@@ -45,6 +45,7 @@
 import java.util.concurrent.Callable;
 import java.util.jar.Manifest;
 import java.util.logging.Level;
+import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 /**
@@ -988,17 +989,7 @@
     if (language.getChunker() != null) {
       language.getChunker().addChunkTags(aTokens);
     }
-    int numTokens = aTokens.size();
-    int posFix = 0; 
-    for (int i = 1; i < numTokens; i++) {
-      aTokens.get(i).setWhitespaceBefore(aTokens.get(i - 1).isWhitespace());
-      aTokens.get(i).setStartPos(aTokens.get(i).getStartPos() + posFix);
-      if (!softHyphenTokens.isEmpty() && softHyphenTokens.get(i) != null) {
-        aTokens.get(i).addReading(language.getTagger().createToken(softHyphenTokens.get(i), null));
-        posFix += softHyphenTokens.get(i).length() - aTokens.get(i).getToken().length();
-      }
-    }
-        
+
     AnalyzedTokenReadings[] tokenArray = new AnalyzedTokenReadings[tokens.size() + 1];
     AnalyzedToken[] startTokenArray = new AnalyzedToken[1];
     int toArrayCount = 0;
@@ -1012,6 +1003,23 @@
       startPos += posTag.getToken().length();
     }
 
+    int numTokens = aTokens.size();
+    int posFix = 0; 
+    for (int i = 0; i < numTokens; i++) {
+      if( i > 0 ) {
+        aTokens.get(i).setWhitespaceBefore(aTokens.get(i - 1).isWhitespace());
+        aTokens.get(i).setStartPos(aTokens.get(i).getStartPos() + posFix);
+      }
+      if (!softHyphenTokens.isEmpty() && softHyphenTokens.get(i) != null) {
+        // addReading() modifies a token if last token is longer - need store first
+        String oldToken = aTokens.get(i).getToken();
+        AnalyzedToken newToken = language.getTagger().createToken(softHyphenTokens.get(i), null);
+        aTokens.get(i).addReading(newToken);
+        posFix += softHyphenTokens.get(i).length() - oldToken.length();
+      }
+    }
+        
+
     // add additional tags
     int lastToken = toArrayCount - 1;
     // make SENT_END appear at last not whitespace token
@@ -1037,9 +1045,10 @@
       return ignoredCharsTokens;
     }
     for (int i = 0; i < tokens.size(); i++) {
-      if (ignoredCharacterRegex.matcher(tokens.get(i)).find()) {
+      Matcher matcher = ignoredCharacterRegex.matcher(tokens.get(i));
+      if (matcher.find()) {
         ignoredCharsTokens.put(i, tokens.get(i));
-        tokens.set(i, ignoredCharacterRegex.matcher(tokens.get(i)).replaceAll(""));
+        tokens.set(i, matcher.replaceAll(""));
       }
     }
     return ignoredCharsTokens;
diff --git a/languagetool-core/src/test/java/org/languagetool/tagging/MorfologikTaggerTest.java b/languagetool-core/src/test/java/org/languagetool/tagging/MorfologikTaggerTest.java
index 8b6cd71..a3057fc 100644
--- a/languagetool-core/src/test/java/org/languagetool/tagging/MorfologikTaggerTest.java
+++ b/languagetool-core/src/test/java/org/languagetool/tagging/MorfologikTaggerTest.java
@@ -19,7 +19,11 @@
 package org.languagetool.tagging;
 
 import org.junit.Test;
+import org.languagetool.AnalyzedSentence;
+import org.languagetool.JLanguageTool;
+import org.languagetool.language.Demo;
 
+import java.io.IOException;
 import java.net.URL;
 import java.util.List;
 
@@ -52,4 +56,14 @@
     assertThat(noResult.size(), is(0));
   }
 
+  @Test
+  public void testPositionWithIgnoredChars() throws IOException {
+    JLanguageTool languageTool = new JLanguageTool(new Demo());
+    String text = "t\u00ADo test";
+    AnalyzedSentence sent = languageTool.getRawAnalyzedSentence(text);
+    assertNotNull(sent.getTokens()[3].getToken());
+    assertEquals("SENT_END", sent.getTokens()[3].getAnalyzedToken(0).getPOSTag());
+    assertEquals(text.indexOf("test"), sent.getTokens()[3].getStartPos());
+  }
+
 }
